/**
* This sample shows how to calibrate the analog outputs
**/
#include "stdafx.h"
#include <math.h>

#define VALUE_OFFSET_BIPOLAR	    0
#define VALUE_OFFSET_UNIPOLAR	    0.01
#define VALUE_GAIN_BIPOLAR		    9.9951
#define FINE_VALUE_GAIN_BIPOLAR     -9.9951
#define VALUE_GAIN_UNIPOLAR		    9.9951
#define FINE_VALUE_GAIN_UNIPOLAR    9.9951
#define POTI_MIDSCALE_VALUE         128 /* midscale position = 0x80 (see datasheet of the AD5262) */
#define NR_OF_POTIS                 4   /* There are four potis: 0 = Bipolar Offset, 1 = Unipolar Offset, 2 = Unipolar Gain, 3 = Bipolar Gain */

/**
* Get the value of the channel 0
* The value is read 1000 times and then an arithmetic mean value is computed
* @param polarity The polarity, used for the measurement: 0 unipolar, 1 bipolar
* @param channel The analog input channel which shall be used for the acquisition
* @param meanValue (= measurement result in LSB (= digital value))
**/
INT GetMeanValue ( HANDLE boardHandle, BYTE polarity, BYTE channel, DWORD * meanValue )
{
	INT returnValue = 0;
	WORD value = 0;
	DOUBLE d_meanVal = 0.0;
	int j = 0;

	/* initialization */
	*meanValue = 0;

	/* Read the current analog value */
	for (j = 0; j < 1000; j++)
	{
		returnValue = i_PCI3116_Read1AnalogInput(
			boardHandle,
			channel,	// channel
			0,			// no gain
			polarity,	// polarity
			1000,		// 1ms converting time
			0,			// no interrupts
			0,			// no software trigger
			0,			// no external trigger
			&value);
		if (returnValue != 0)
		{
			printf("Error %d in i_PCI3116_Read1AnalogInput\n", returnValue);
			return -1;
		}
		*meanValue = *meanValue + value;
	}

	/* get the mean value */
	d_meanVal = (double)(*meanValue) / 1000.0;
	*meanValue = *meanValue / 1000;
	*meanValue = ((d_meanVal - (double)(*meanValue)) < 0.5) ? *meanValue : *meanValue + 1; // round up if necessary

	return 0;
}

/**
* Calculates the voltage difference of "measured voltage" - "nominal voltage"
* @param meanValue (= measurement result in LSB (= digital value))
* @param measuredVoltage Is calculated with the measurement result (meanValue) and then returned.
* @param nominalVoltage The voltage that the user has set on the input channel (in [V])
* @param diffVoltage The calculated voltage difference (in [V])
* @param polarity The polarity, used for the measurement: 0 unipolar, 1 bipolar
**/
INT GetVoltageDifference( DWORD meanValue, double* measuredVoltage, double nominalVoltage, double* diffVoltage, BYTE polarity )
{    
    // compute the analog voltage value
    if (polarity == 0)
    {   // UNIPOLAR
        *measuredVoltage = ((double)meanValue / 65535.0) * 10.0;
    }
    else if (polarity == 1)
    {   // BIPOLAR        
        *measuredVoltage = ((double)meanValue / 65535.0 )* 20.0 - 10.0;
    }
    else
    {
        printf("Error in GetVoltageDifference(...): Polarity value is unknown !\n");
        *diffVoltage = 0.0;
        return -1;
    }

    // compute the voltage difference
    *diffVoltage = *measuredVoltage - nominalVoltage;
    
    return 0;
}

/**
* Reset all potis to their midscale value
**/
INT ResetPotisToMidscale ( HANDLE boardHandle )
{
    BYTE i;
    INT returnValue = 0;
    
    for (i=0; i<NR_OF_POTIS; i++)
    {
        // write the midscale value to the corresponding poti
        returnValue = i_PCI3116_WritePotis(boardHandle, i, POTI_MIDSCALE_VALUE);
        if (returnValue != 0)
        {            
            printf("Error in ResetPotisToMidscale(...): i_PCI3116_WritePotis(...) returned: %d\n", returnValue);
            return -1;
        }

        Sleep(100);
    }
    
    return 0;
}

/**
* calibrate a poti
* @param potiId The id of the poti to calibrated
* @param voltageToSet The voltage that the user must set on the input channel (in V)
* @param polarity 0 unipolar, 1 bipolar
* @param value The correct value of the poti
**/
INT CalibratePoti ( HANDLE boardHandle, BYTE potiId, double voltageToSet, BYTE polarity, DWORD * value )
{
	INT returnValue = 0;
	DWORD min = 0;
	DWORD max = 255;
	DWORD old = 0;
	int iteration = 0;

	/* store the value, to choose the lowest one at the end of the dichotomie */
	double voltdiffs[255] = {0};
	DWORD potivalues[255] = {0};

	/* memory initialization */
	memset( voltdiffs, 0, 255 * sizeof(double) );
	memset( potivalues, 0, 255 * sizeof(DWORD) );

	printf("\nPlease set a voltage of %.05f V\n", voltageToSet);
	//if ( channel == 0 )
	//{
		printf("Between pin 20 (+) and 9 (-)\n");
	//}
	//else
	//{
	//	printf("Between pin 1 (+) and 9 (-)\n");
	//}
	printf("then press enter.\n");
	while (_getch() != VK_RETURN);

	/* dichotomic algorithm */
	for(;;)
	{
		DWORD meanValue = 0;
		double measuredVoltage = 0.0;
		double diff = 0.0;

		/* write the value of the poti */
		returnValue = i_PCI3116_WritePotis(boardHandle, potiId, *value);
		if (returnValue != 0)
		{			
			printf("Error in CalibratePoti(...): i_PCI3116_WritePotis(...) returned: %d\n", returnValue);
			return -1;
		}
		Sleep(100);

		/* save the value */
		old = *value;

		/* get the value of the analog input */
		returnValue = GetMeanValue( boardHandle, polarity, 0, &meanValue);
		if (returnValue != 0)
		{
			printf("Error %d in GetMeanValue(...)\n", returnValue);
			return -2;
		}
		
		// calculate the voltage difference
		GetVoltageDifference(meanValue, &measuredVoltage, voltageToSet, &diff, polarity);
		
		/* store the diff */
		if ( diff < 0 )
		{
			voltdiffs[iteration] = -diff;
		}
		else
		{
			voltdiffs[iteration] = diff;
		}
		potivalues[iteration] = *value;

		/* display current status */
		printf("Voltage : %.06f \t difference : %.06f \t Poti value : %d \n", measuredVoltage, diff, *value);

		/* ok ? */
		if ( diff == 0 )
		{
			return 0;
		}

		/** 
		* compute the new poti value, using dichotomie 
		* unipolar gain needs to be inverted
		*/
		if ( potiId != 2 )
		{
			if ( diff < 0 )
			{
				/* have to decrement */
				max = *value;
			}
			else
			{
				/* have to increment */
				min = *value;
			}
		}
		else
		{
			if ( diff > 0 )
			{
				/* have to decrement */
				max = *value;
			}
			else
			{
				/* have to increment */
				min = *value;
			}
		}
		*value = ((max - min) / 2) + min;

		/* end */
		if ( old == *value )
		{
			double mindiff = voltdiffs[0];
			DWORD potiminvalue = potivalues[0];
			/* loop through the array to get the lowest diff */
			for ( int j = 1; j < iteration; j++ )
			{
				/* lower ? */
				if ( voltdiffs[j] < mindiff )
				{
					mindiff = voltdiffs[j];
					potiminvalue = potivalues[j];
				}
			}

			/* set the value */
			*value = potiminvalue;

			printf("Poti value chosen : %d, difference is : %.06f\n", *value, mindiff);

			break;
		}

		/* next iteration */
		iteration++;
	}

	return 0;
}

/**
* Do a fine calibration, if the selected poti value is not yet correct already
* @param potiId The id of the poti to calibrated
* @param voltageToSet The voltage that the user must set on the input channel (in [V])
* @param tolerance The tolerance in [V], that must be met
* @param polarity 0 unipolar, 1 bipolar
* @param value The correct value of the poti
**/
INT FineCalibration ( HANDLE boardHandle, BYTE potiId, double voltageToSet, double tolerance, BYTE polarity, DWORD * value )
{	
    DWORD meanValue = 0; 
    double measuredVoltage = 0.0;
    double diff = 0.0;
    DWORD poti_val = *value;
    BYTE no_of_cycles = 0;
    INT returnValue = 0;

    printf("\nPlease set a voltage of %.05f V\n", voltageToSet);
    printf("Between pin 20 (+) and 9 (-)\n");	
	printf("then press enter.\n");
	while (_getch() != VK_RETURN);
    
    // do the fine calibration, using the given control voltage
    do
    {        
        // get the value of the analog input
        returnValue = GetMeanValue( boardHandle, polarity, 0, &meanValue);
        if (returnValue != 0)
        {
            printf("Error in FineCalibration(...)\n");
            return -1;
        }
        
        // calculate the voltage difference
        returnValue = GetVoltageDifference(meanValue, &measuredVoltage, voltageToSet, &diff, polarity);
        if (returnValue != 0)
        {
            printf("Error in FineCalibration(...)\n");
            return -2;
        }
        
        // display current status
		printf("Voltage : %.06f \t difference : %.06f \t Poti value : %d \n", measuredVoltage, diff, poti_val);
        
        // Check, if the measured voltage is in the tolerance. If so, no correction is needed.
        if (fabs(diff) < tolerance) break;
        
        // check the polarity
        if (polarity == 0) // UNIPOLAR
        {   
            // Check, if the corresponding poti must be increased or decreased (JB: 2014-05-06)
            if (diff >= 0) // the difference is positive
            {
                // decrease the poti value
                poti_val = (poti_val - 1) < 0 ? 0 : (poti_val - 1);                
            }
            else // the difference is negative
            {   
                // increase the poti value
                poti_val = (poti_val + 1) < 256 ? (poti_val + 1) : 255; 
            }
        }
        else if (polarity == 1) // BIPOLAR
        {
            // Check, if the corresponding poti must be increased or decreased (JB: 2014-05-06)
            if ((diff*voltageToSet) >= 0) // positive sign 
            {        
                // increase the poti value
                poti_val = (poti_val + 1) < 256 ? (poti_val + 1) : 255;                
            }
            else // negative sign 
            {
                // decrease the poti value
                poti_val = (poti_val - 1) < 0 ? 0 : (poti_val - 1); 
            }
        }
        else
        {
            printf("Error in FineCalibration(...): Polarity value is unknown !\n");
        }

        // write the new value to the poti ...
        returnValue = i_PCI3116_WritePotis(boardHandle, potiId, poti_val); 
        if (returnValue != 0)
        {
            printf("Error in FineCalibration(...): i_PCI3116_WritePotis(...) returned: %d\n", returnValue);
            return -3;
        }
        Sleep(100);
        
        // increase the cycle counter
        no_of_cycles++; 
    } while (no_of_cycles < 10); // if the voltage is still not in tolerance after 10 cycles then cancel the fine calibration
    
    // Has the fine calibration been successful ?
    if (no_of_cycles == 10)
    {
        printf("Error in FineCalibration(...): The fine calibration failed !");
        return -4;
    }
    
    // return the new poti value
    *value = poti_val;
    
    return 0;
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
    DWORD valuepotiunipolaroffset = 127;
	DWORD valuepotiunipolargain = 127;
	DWORD valuepotibipolaroffset = 127;
	DWORD valuepotibipolargain = 127;

	// open a board
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	// set single-ended mode
	returnValue = i_PCI3116_SetAnalogInputMode(boardHandle, 1);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_SetAnalogInputMode\n", returnValue);
		i_PCI3116_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	// now do the calibration ...
    do
	{
        // before starting the calibration, reset all potis to their midscale value
        returnValue = ResetPotisToMidscale(boardHandle);
        if (returnValue != 0) break;
        
		// first of all, do the bipolar offset calibration (POTI: 0) using the channel 0
        printf("\nBipolar offset calibration:");
		returnValue = CalibratePoti ( boardHandle, 0, VALUE_OFFSET_BIPOLAR, PCI3116_OUTPUT_MODE_BIPOLAR, &valuepotibipolaroffset );
		if (returnValue != 0) break;
		
		// now do the unipolar offset calibration (POTI: 1) using the channel 0
        printf("\nUnipolar offset calibration:");
		returnValue = CalibratePoti ( boardHandle, 1, VALUE_OFFSET_UNIPOLAR, PCI3116_OUTPUT_MODE_UNIPOLAR, &valuepotiunipolaroffset );
		if (returnValue != 0) break;
				
        // now do the unipolar gain calibration (POTI: 2) using the channel 0
        printf("\nUnipolar gain calibration:");
		returnValue = CalibratePoti ( boardHandle, 2, VALUE_GAIN_UNIPOLAR, PCI3116_OUTPUT_MODE_UNIPOLAR, &valuepotiunipolargain );
		if (returnValue != 0) break;
        printf("\nUnipolar gain fine calibration:");
        returnValue = FineCalibration ( boardHandle, 2, FINE_VALUE_GAIN_UNIPOLAR, 0.0002, PCI3116_OUTPUT_MODE_UNIPOLAR, &valuepotiunipolargain );
        if (returnValue != 0) break;
		
        // finally do the bipolar gain calibration (POTI: 3) using the channel 0
        printf("\nBipolar gain calibration:");
		returnValue = CalibratePoti ( boardHandle, 3, VALUE_GAIN_BIPOLAR, PCI3116_OUTPUT_MODE_BIPOLAR, &valuepotibipolargain );
		if (returnValue != 0) break;
        printf("\nBipolar gain fine calibration:");
        returnValue = FineCalibration ( boardHandle, 3, FINE_VALUE_GAIN_BIPOLAR, 0.0004, PCI3116_OUTPUT_MODE_BIPOLAR, &valuepotibipolargain );
        if (returnValue != 0) break;
        		
		/* display the configuration :*/
        printf("\nFinal poti values:\n");
		printf("Poti 0 : %d \n", (int) valuepotibipolaroffset);
		printf("Poti 1 : %d \n", (int) valuepotiunipolaroffset);
		printf("Poti 2 : %d \n", (int) valuepotiunipolargain);
		printf("Poti 3 : %d \n", (int) valuepotibipolargain);
        break;
	} while (1);
    // Check, if an error has been returned. If so, then exit.
    if (returnValue != 0)
    {
        i_PCI3116_CloseBoard(boardHandle);
        _getch();
        return -1;
    }

	// save the values in the eeprom
	returnValue = i_PCI3116_SavePotis(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3116_SavePotis\n", returnValue);
		i_PCI3116_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3116_SavePotis ok.\n");
	}

	printf("Hit a key to exit this program...\n");
	_getch();

	return 0;
}
