using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TimerInterrupt
{
    public partial class TimerInterrupt : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCI3116_NET.PCI3116 board;

        /// <summary>
        /// Some internal value. Store the number of interrupt per component
        /// </summary>
        private int timer1InterruptNb = 0;
        private int timer2InterruptNb = 0;
        private int timer3InterruptNb = 0;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public TimerInterrupt()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TimerInterrupt_Load(object sender, EventArgs e)
        {
            int ret = 0;

            // Open the first board of the system
            this.board = PCI3116_NET.PCI3116.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                this.Close();
                return;
            }

            // Stop the components. No check since if nothing is started the function will failed
            for (byte i = 0; i < 3; i++)
            {
                this.board.StopCounterTimerWatchdog(i);
            }

            // Listen to the counter timer watchdog interruption
            this.board.CounterTimerWatchdogEventReceived += new PCI3116_NET.PCI3116.CounterTimerWatchdogEventHandler(board_CounterTimerWatchdogEventReceived);
        }

        /// <summary>
        /// Counter timer watchdog interruption
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_CounterTimerWatchdogEventReceived(object source, PCI3116_NET.CounterTimerWatchdogEventArgs args)
        {
            if (args.InterruptMask == 0)
            {
                this.timer1InterruptNb++;
            }
            else if (args.InterruptMask == 1)
            {
                this.timer2InterruptNb++;
            }
            else if (args.InterruptMask == 2)
            {
                this.timer3InterruptNb++;
            }
        }

        /// <summary>
        /// Display timer for timer 1 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadTimerValue(0, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadTimerValue");
                return;
            }
            timer1Value.Text = value.ToString();
            timer1Interrupt.Text = this.timer1InterruptNb.ToString();
        }

        /// <summary>
        /// Click on the start button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Start_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Initialize the component as a timer
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(1, // timer function
                0, // index
                1, // use interrupt
                5000, // reload value
                1, // time base set to ms
                0, // mode 2
                0, // not used
                0, // not used
                0 // not used
                )) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            timer1Start.Enabled = false;
            timer1Stop.Enabled = true;
            timer1Trigger.Enabled = true;
            // Start the display timer
            timer1DisplayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the timer
            if ((ret = this.board.StopCounterTimerWatchdog(0)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            timer1Start.Enabled = true;
            timer1Stop.Enabled = false;
            timer1Trigger.Enabled = false;
            // Stop the display timer
            timer1DisplayTimer.Stop();
        }

        /// <summary>
        /// Click on the trigger button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Trigger
            if ((ret = this.board.TriggerCounterTimerWatchdog(0)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Click on the start button for the timer 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer2Start_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Initialize the component as a timer
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(1, // timer function
                1, // index
                1, // use interrupt
                5000, // reload value
                1, // time base set to ms
                0, // mode 2
                0, // not used
                0, // not used
                0 // not used
                )) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            timer2Start.Enabled = false;
            timer2Stop.Enabled = true;
            timer2Trigger.Enabled = true;
            // Start the display timer
            timer2DisplayTimer.Start();
        }

        /// <summary>
        /// Display timer for timer 2 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer2DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadTimerValue(1, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadTimerValue");
                return;
            }
            timer2Value.Text = value.ToString();
            timer2Interrupt.Text = this.timer2InterruptNb.ToString();
        }

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TimerInterrupt_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;
            // Stop the timers
            for (byte i = 0; i < 3; i++)
            {
                this.board.StopCounterTimerWatchdog(i);
            }
            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }

        /// <summary>
        /// Click on the stop button for the timer 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer2Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the timer
            if ((ret = this.board.StopCounterTimerWatchdog(1)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            timer2Start.Enabled = true;
            timer2Stop.Enabled = false;
            timer2Trigger.Enabled = false;
            // Stop the display timer
            timer2DisplayTimer.Stop();
        }

        /// <summary>
        /// Click on the trigger button for the timer 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer2Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Trigger
            if ((ret = this.board.TriggerCounterTimerWatchdog(1)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Click on the start button of the timer 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer3Start_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Initialize the component as a timer
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(1, // timer function
                2, // index
                1, // use interrupt
                5000, // reload value
                1, // time base set to ms
                0, // mode 2
                0, // not used
                0, // not used
                0 // not used
                )) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            timer3Start.Enabled = false;
            timer3Stop.Enabled = true;
            timer3Trigger.Enabled = true;
            // Start the display timer
            timer3DisplayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button of the timer 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer3Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the timer
            if ((ret = this.board.StopCounterTimerWatchdog(2)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            timer3Start.Enabled = true;
            timer3Stop.Enabled = false;
            timer3Trigger.Enabled = false;
            // Stop the display timer
            timer3DisplayTimer.Stop();
        }

        /// <summary>
        /// Click on the trigger button of the timer 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer3Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Trigger
            if ((ret = this.board.TriggerCounterTimerWatchdog(2)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Display timer of timer 3 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer3DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadTimerValue(2, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadTimerValue");
                return;
            }
            timer3Value.Text = value.ToString();
            timer3Interrupt.Text = this.timer3InterruptNb.ToString();
        }
    }
}