using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WatchdogInterrupt
{
    public partial class WatchdogInterruptForm : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCI3116_NET.PCI3116 board;

        /// <summary>
        /// Some internal value. Store the number of interrupt per component
        /// </summary>
        private int watchdog2InterruptNb = 0;
        private int watchdog3InterruptNb = 0;

        /// <summary>
        /// Useful array to access all the checkboxes
        /// </summary>
        private CheckBox[] outputCheckBoxes;

        /// <summary>
        /// Enables to lock the events
        /// </summary>
        private bool locked;

        /// <summary>
        /// Useful array
        /// </summary>
        private TrackBar[] outputTrackBar;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public WatchdogInterruptForm()
        {
            InitializeComponent();

            // Create the useful array
            this.outputTrackBar = new TrackBar[4];
            this.outputTrackBar[0] = this.outputTrack0;
            this.outputTrackBar[1] = this.outputTrack1;
            this.outputTrackBar[2] = this.outputTrack2;
            this.outputTrackBar[3] = this.outputTrack3;
            this.outputCheckBoxes = new CheckBox[4];
            this.outputCheckBoxes[0] = this.output0CheckBox;
            this.outputCheckBoxes[1] = this.output1CheckBox;
            this.outputCheckBoxes[2] = this.output2CheckBox;
            this.outputCheckBoxes[3] = this.output3CheckBox;
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void WatchdogInterruptForm_Load(object sender, EventArgs e)
        {
            int ret = 0;

            // Open the first board of the system
            this.board = PCI3116_NET.PCI3116.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                this.Close();
                return;
            }

            // Stop the components. No check since if nothing is started the function will failed
            this.board.StopCounterTimerWatchdog(1);
            this.board.StopCounterTimerWatchdog(2);

            // Disable output memory
            if ((ret = this.board.SetDigitalOutputMemoryOff()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetDigitalOutputMemoryOff");
                this.Close();
                return;
            }

            // Initialize the analog outputs to 0V
            for (byte i = 0; i < 4; i++)
            {
                int returnValue = 0;
                // We use bipolar mode
                // 0x7ff is 0V
                // 0x0 is -10V
                // 0xfff is 10V
                if ((returnValue = this.board.Write1AnalogOutput(i, 1, 0x7ff)) != 0)
                {
                    MessageBox.Show("Write1AnalogOutput error : " + returnValue.ToString());
                    this.Close();
                    return;
                }
            }

            // Listen to the counter timer watchdog interruption
            this.board.CounterTimerWatchdogEventReceived += new PCI3116_NET.PCI3116.CounterTimerWatchdogEventHandler(board_CounterTimerWatchdogEventReceived);

            // Initialize graphics
            this.InitTrackBar();
        }

        /// <summary>
        /// Initialize the track bar
        /// </summary>
        private void InitTrackBar()
        {
            for (int i = 0; i < 4; i++)
            {
                this.outputTrackBar[i].Minimum = -10;
                this.outputTrackBar[i].Maximum = 10;
                this.outputTrackBar[i].TickFrequency = 1;
                this.outputTrackBar[i].SmallChange = 1;
                this.outputTrackBar[i].LargeChange = 2;
                this.outputTrackBar[i].ValueChanged += new EventHandler(WatchdogInterruptForm_ValueChanged);
            }
        }

        /// <summary>
        /// Compute the value that must be written in the board
        /// </summary>
        /// <param name="value">The track bar value: -10 to +10</param>
        /// <returns></returns>
        private ushort ComputeOutputValue(int value)
        {
            value += 10;
            return (ushort)((value * 0xfff) / 20);
        }

        /// <summary>
        /// Analog output value is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void WatchdogInterruptForm_ValueChanged(object sender, EventArgs e)
        {
            for (byte i = 0; i < 4; i++)
            {
                if (sender == this.outputTrackBar[i])
                {
                    int returnValue = 0;
                    if ((returnValue = this.board.Write1AnalogOutput(i, 1, ComputeOutputValue(this.outputTrackBar[i].Value))) != 0)
                    {
                        MessageBox.Show("Write1AnalogOutput error : " + returnValue.ToString());
                        this.Close();
                        return;
                    }
                    return;
                }
            }
        }

        /// <summary>
        /// Called when we get an interrupt
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_CounterTimerWatchdogEventReceived(object source, PCI3116_NET.CounterTimerWatchdogEventArgs args)
        {
            if (args.InterruptMask == 1)
            {
                this.watchdog2InterruptNb++;
            }
            else if (args.InterruptMask == 2)
            {
                this.watchdog3InterruptNb++;
            }
        }        

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void WatchdogInterruptForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;
            // Stop the 2 watchdogs
            this.board.StopCounterTimerWatchdog(1);
            this.board.StopCounterTimerWatchdog(2);
            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }

        /// <summary>
        /// Start watchdog 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogStart2_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Initialize the component as a timer
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(2, // watchdog function
                1, // index
                1, // use interrupt
                5000, // reload value
                1, // time base set to ms
                0, // not used
                0, // not used
                0, // not used
                0 // not used
                )) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            this.watchdogStart2.Enabled = false;
            this.watchdogStop2.Enabled = true;
            this.watchdogTrigger2.Enabled = true;
            // Start the display timer
            watchdogDisplayTimer2.Start();
        }

        /// <summary>
        /// Display timer ticks for watchdog 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogDisplayTimer2_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadWatchdogValue(1, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadWatchdogValue");
                return;
            }
            watchdogValue2.Text = value.ToString();
            watchdogInterrupt2.Text = this.watchdog2InterruptNb.ToString();
        }

        /// <summary>
        /// Trigger watchdog 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogTrigger2_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Trigger
            if ((ret = this.board.TriggerCounterTimerWatchdog(1)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Stop watchdog 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogStop2_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the timer
            if ((ret = this.board.StopCounterTimerWatchdog(1)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            this.watchdogStart2.Enabled = true;
            this.watchdogStop2.Enabled = false;
            this.watchdogTrigger2.Enabled = false;
            // Stop the display timer
            watchdogDisplayTimer2.Stop();
        }

        /// <summary>
        /// Change the state of the output memory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void outputMemoryCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            int returnValue = 0;

            if (outputMemoryCheckBox.Checked == true)
            {
                // Set to on
                if ((returnValue = board.SetDigitalOutputMemoryOn()) != 0)
                {
                    MessageBox.Show("Error while setting digital output memory to on : " + returnValue);
                }
            }
            else
            {
                // Set to off
                if ((returnValue = board.SetDigitalOutputMemoryOff()) != 0)
                {
                    MessageBox.Show("Error while setting digital output memory to off : " + returnValue);
                }
            }
        }

        /// <summary>
        /// Update the output values
        /// </summary>
        private void UpdateOutputValues()
        {
            byte outputStatus = 0;
            int returnValue = 0;

            // Get the current status of the outputs
            if ((returnValue = board.Get4DigitalOutputsStatus(ref outputStatus)) != 0)
            {
                MessageBox.Show("Error while getting status of the 4 digital outputs : " + returnValue);
                return;
            }

            // Lock
            locked = true;

            // Change the status of the checkboxes
            for (int i = 0; i < 4; i++)
            {
                if ((uint)(outputStatus & (1 << i)) == (uint)(1 << i))
                {
                    this.outputCheckBoxes[i].Checked = true;
                }
                else
                {
                    this.outputCheckBoxes[i].Checked = false;
                }
            }

            // Unlock
            locked = false;
        }

        /// <summary>
        /// Change the checkbox value
        /// </summary>
        /// <param name="index"></param>
        private void CheckBoxChanged(int index)
        {
            int returnValue = 0;

            // Changment due to update value function ?
            if (locked == true)
            {
                return;
            }

            if (this.outputCheckBoxes[index].Checked == true)
            {
                // Set to on
                if ((returnValue = board.Set4DigitalOutputsOn((byte)(1 << index))) != 0)
                {
                    MessageBox.Show("Error while setting 4 digital outputs to on : " + returnValue);
                }
            }
            else
            {
                // Set to off

                // Check the status of the output memory
                if (outputMemoryCheckBox.Checked == true)
                {
                    if ((returnValue = board.Set4DigitalOutputsOff((byte)(1 << index))) != 0)
                    {
                        MessageBox.Show("Error while setting 4 digital outputs to off : " + returnValue);
                    }
                }
            }

            // Update values
            this.UpdateOutputValues();
        }

        private void output0CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(0);
        }

        private void output1CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(1);
        }

        private void output2CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(2);
        }

        private void output3CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(3);
        }

        /// <summary>
        /// Start watchdog 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogStart3_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Initialize the component as a timer
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(2, // watchdog function
                2, // index
                1, // use interrupt
                5000, // reload value
                1, // time base set to ms
                0, // not used
                0, // not used
                0, // not used
                0 // not used
                )) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            this.watchdogStart3.Enabled = false;
            this.watchdogStop3.Enabled = true;
            this.watchdogTrigger3.Enabled = true;
            // Start the display timer
            watchdogDisplayTimer3.Start();
        }

        /// <summary>
        /// Stop watchdog 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogStop3_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the timer
            if ((ret = this.board.StopCounterTimerWatchdog(2)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            this.watchdogStart3.Enabled = true;
            this.watchdogStop3.Enabled = false;
            this.watchdogTrigger3.Enabled = false;
            // Stop the display timer
            watchdogDisplayTimer3.Stop();
        }

        /// <summary>
        /// Trigger watchdog 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogTrigger3_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Trigger
            if ((ret = this.board.TriggerCounterTimerWatchdog(2)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Display timer ticks for watchdog 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogDisplayTimer3_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadWatchdogValue(2, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadWatchdogValue");
                return;
            }
            watchdogValue3.Text = value.ToString();
            watchdogInterrupt3.Text = this.watchdog3InterruptNb.ToString();

            // Update output status
            this.UpdateOutputValues();
        }
    }
}